package meerkat.voting.gui.ui.controllersFX;

/**
 * Created by Laura on 12/16/2016.
 */
import javafx.fxml.FXMLLoader;
import meerkat.voting.gui.ui.GraphicalUI;
import meerkat.voting.gui.ui.uicommands.UICommand;

import java.io.IOException;

/**
 * Utility class for controlling navigation between vistas.
 *
 * All methods on the navigator are static to facilitate
 * simple access from anywhere in the application.
 */
public class VistaNavigator {

    /**
     * Convenience constants for fxml layouts managed by the navigator.
     */
    public static final String MAIN    = "/views/main.fxml";
    public static final String LOADING    = "/views/loading.fxml";
    public static final String VISTA_1 = "/views/starting_session.fxml";
    public static final String VISTA_2 = "/views/vista2.fxml";
    public static final String CHANNEL_CHOICE = "/views/channel_choice.fxml";

    /** The main application layout controller. */
    private static MainController mainController;
    public static GraphicalUI uiThread;


    /**
     * Stores the main controller for later use in navigation tasks.
     *
     * @param mainController the main application layout controller.
     */
    public static void setMainController(MainController mainController) {
        VistaNavigator.mainController = mainController;
    }

    public static MainController getMainController() {
        return VistaNavigator.mainController;
    }

    /**
     * Stores the thread that controls the UI.
     *
     * @param ui the thread that launched the fx application.
     */
    public static void setUIThread(GraphicalUI ui) {
        VistaNavigator.uiThread = ui;
    }

    /**
     * Stores the command that is requested to be executed.
     *
     * @param command the command that needs to be executed.
     */
    public static void setCurrentCommand(UICommand command) {
        VistaNavigator.mainController.setCurrentCommand(command);
    }

    /**
     * Returns the command that is requested to be executed.
     */
    public static UICommand getCurrentCommand() {
        return VistaNavigator.mainController.getCurrentCommand();
    }

    /**
     * Loads the vista specified by the fxml file into the
     * vistaHolder pane of the main application layout.
     *
     * Previously loaded vista for the same fxml file are not cached.
     * The fxml is loaded anew and a new vista node hierarchy generated
     * every time this method is invoked.
     *
     * A more sophisticated load function could potentially add some
     * enhancements or optimizations, for example:
     *   cache FXMLLoaders
     *   cache loaded vista nodes, so they can be recalled or reused
     *   allow a user to specify vista node reuse or new creation
     *   allow back and forward history like a browser
     *
     * @param fxml the fxml file to be loaded.
     */
    public static void loadVista(String fxml) {
        try {
            mainController.setVista(
                    FXMLLoader.load(VistaNavigator.class.getResource(fxml)
                    )
            );
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}